/*
	Copyright (c) 2004-2009, The Dojo Foundation All Rights Reserved.
	Available via Academic Free License >= 2.1 OR the modified BSD license.
	see: http://dojotoolkit.org/license for details
*/


if(!dojo._hasResource["fbfwca.Grid"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["fbfwca.Grid"] = true;
dojo.provide("fbfwca.Grid");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.grid.DataGrid");
dojo.require("dojox.grid.cells");
dojo.require("dijit.Toolbar");
dojo.require("ibm.tivoli.tip.dijit.TIPButton");
dojo.require("ibm.tivoli.tip.dijit.TIPTableSearchField_mod");
dojo.require("fbfwca.ajax");

// NLS
dojo.requireLocalization("nls", "general", null, "");
dojo.requireLocalization("ibm.tivoli.tip.dijit", "resources", null, "ROOT,ar,cs,da,de,en,en-us,es,fi,fr,he,hr,hu,it,ja,ko,nl,no,pl,pt-br,ro,ru,sl,sv,tr,zh-cn,zh-tw");

dojo.declare("fbfwca.Grid", [dijit._Widget, dijit._Templated], {

	widgetsInTemplate: true,
	
	templateString:"<table width=\"100%\" border=\"0\" cellpadding=\"0\" cellspacing=\"0\" >\r\n\t<tr class=\"tableToolbar\" >\r\n\t\t<td class=\"TIPTableToolbarArea\">\r\n\t\t\t\t<div dojoType=\"dijit.Toolbar\" dojoAttachPoint=\"toolbar\" class=\"tableToolbarArea\" ></div>\r\n\t\t</td>\r\n\t\t<td class=\"TIPTableSearchFieldArea\">\r\n\t    <div dojoType=\"ibm.tivoli.tip.dijit.TIPTableSearchField_mod\" dojoAttachPoint=\"searchField\"></div>\r\n    </td>\r\n\t</tr>\r\n\t<tr>\r\n\t\t<td colspan=\"2\" valign=\"top\">\r\n\t\t<div dojoAttachPoint=\"gridContainer\" ></div>\r\n\t\t</td>\r\n\t</tr>\r\n\t<tr>\r\n\t\t<td colspan=\"2\" valign=\"top\">\r\n\t\t<table width=\"100%\" cellspacing=\"0\" cellpadding=\"0\" border=\"0\">\r\n\t\t\t<tbody>\r\n\t\t\t\t<tr class=\"tip-table-footer\" >\r\n\t\t\t\t\t<td class=\"tip-table-footer-data\" nowrap=\"nowrap\" >\r\n\t\t\t\t\t\t<div dojoType=\"dijit.Toolbar\" class=\"tableToolbarArea\" ></div>\r\n\t\t\t\t\t</td>\r\n\t\t\t\t\t<td class=\"tip-table-footer-data\" nowrap=\"nowrap\" valign=\"middle\" >\r\n\t\t\t\t\t\t<div dojoAttachPoint=\"gridRowCount\" ></div>\r\n\t\t\t\t\t</td>\r\n\t\t\t\t</tr>\r\n\t\t\t</tbody>\r\n\t\t</table>\r\n\t\t</td>\r\n\t</tr>\r\n</table>\r\n\r\n",

	gridParms: null,
	multiSelect: true,
	refreshButton: true,  // refresh button
	filterOn: true,       // quick filter
	defaultHeight: "300px",

	/**************************************************************************
	* Constructor
	**************************************************************************/
	constructor: function () {
		this.NLS = dojo.i18n.getLocalization("nls", "general");
		this.resources_ = dojo.i18n.getLocalization("ibm.tivoli.tip.dijit", "resources");	
		this.connections = [];
		this.topics = [];
		this.query = {};
		this.grid = null;
		this.storeURL = "";     // URL to get data
		this.store = null;
		this.layout = null;
		this.createButton = null;
		this.deleteButton = null;
		this.actionsDropDown = null;
		this.autoHeight = true;
	},
	
	/**************************************************************************
	* Cleanup function
	**************************************************************************/
	destroy : function () {
		console.log('fbfwca.Grid: destroy');
  	// Destroy all event connections
		dojo.forEach(this.connections, dojo.disconnect);
  	delete this.connections;

  	// Destroy all subscriptions
  	dojo.forEach(this.topics, dojo.unsubscribe);
  	delete this.topics;

		if (this.toolbar)
		{
			this.toolbar.destroyRecursive();
			delete this.toolbar;
		}

		if (this.grid)
		{
			this.grid.destroyRecursive();
			delete this.grid;
		}
			
		if (this.gridParms.store)
			delete this.gridParms.store;
		
		delete this.gridParms;
		delete this.NLS;	
		delete this.query;
		delete this.storeURL;
		delete this.store;
		delete this.layout;
		delete this.createButton;
		delete this.deleteButton;
		delete this.actionsDropDown;
		delete this.sortIndex;
		delete this.sortAsc;
		delete this.autoHeight;
		
 	},


	/**************************************************************************
	* Called after template has been processed.
	*
	* Lets check to see if a dojox.grid.Grid was passed in on params.  If so,
	* add it to the DOM tree and setup.
	**************************************************************************/
	postCreate: function () {
		//call super
		this.inherited( "postCreate", arguments );
		if ( this.sortIndex == undefined )
			this.sortIndex = 1;
		if ( this.sortAsc == undefined )
			this.sortAsc = "true";

		//-----------------------------
		// Add Toolbar and filter area
		//-----------------------------
    this.initializeToolbar();
    if(dojo.getComputedStyle(dojo.body()).direction == "rtl"){
      this.gridRowCount.setAttribute('align','left');
    }else{
      this.gridRowCount.setAttribute('align','right');
    }

    //-----------------------------
    // Create the grid
    //-----------------------------
		this.gridParms = this.initializeGridParameters();

		// bug in initial sort, fix proposed in 1.4 http://bugs.dojotoolkit.org/ticket/7541     
		this.grid = new dojox.grid.DataGrid(this.gridParms, this.gridContainer);
		this.grid.startup();
		this.grid.setSortIndex(this.sortIndex, this.sortAsc);
		this.grid.resize();
		this.updateFooter (this.grid.rowCount, 0);
		
		if (this.filterOn)
		{
			this.searchField.setTable(this.grid);

			//subscribe event handlers
			this.topics.push (dojo.subscribe (this.searchField.id + "searchInvoked", this, "onSearch"));
		}
		else
		{
			this.searchField.domNode.style.display = "none";
		}
		
		//connect event handlers
		if (this.multiSelect)
		{
			this.connections.push (dojo.connect(this.grid, 'onRowClick', dojo.hitch(this, "onSelectionChanged")));
		}
		
		/*
		this.connections.push (dojo.connect(this.grid, '_onFetchComplete', function() {
						console.log("_onFetchComplete: " + this.rowCount);
						this.updateFooter (this.rowCount, 0);
						this.grid.resize();
				}));
		this.connections.push (dojo.connect(this.grid, '_onFetchError', function() {
						alert("Error was encountered when store was queried: ");
				}));
			*/

	},

	/**************************************************************************
	* Initializes the parameters for the grid.
	*
	**************************************************************************/
	initializeGridParameters: function () {
		if (this.gridParms == null) {
			this.gridParms = {};
		}
/*
		if (this.id) {
			this.gridParms.id = this.id;
		}
*/
		if (this.multiSelect) {
		} else {
			this.gridParms.selectionMode = "none";
		}
		
		if (this.autoHeight)
			this.gridParms.autoHeight = true;
		else if (this.height)
			this.gridParms.height = this.height;
		else
			this.gridParms.height = this.defaultHeight;

		this.gridParms.clientSort = true;
		//this.gridParms.rowsPerPage = 25;
		this.gridParms.query = this.query;
		this.gridParms.structure = this.layout;				

		if (this.store == null )
		{
			console.log("fbfwca.Grid: load store: "+this.storeURL);
			if ( this.multiSelect )
				this.gridParms.store = fbfwca.ajax.getJsonWriteStore(this.storeURL);
			else
				this.gridParms.store = fbfwca.ajax.getJsonReadStore(this.storeURL);
		}
		else
		{
			console.log("fbfwca.Grid: store gotten");
			this.gridParms.store = this.store;
		}
		console.log(this.gridParms);
		return (this.gridParms);
	},
	
	/**************************************************************************
	* Initialize the toolbar with default buttons (like refresh)
	**************************************************************************/
	initializeToolbar: function() 
	{
		var button = null;

		// Add refresh button
	    if (this.refreshButton)
	    {
			button = new ibm.tivoli.tip.dijit.TIPButton(
				{	iconClass: "refreshIcon",
					label: this.resources_.REFRESH,
					showLabel: false,
			 		onClick: dojo.hitch (this, this.onRefresh) 
				});
			this.toolbar.addChild(button);
			this.toolbar.addChild (new dijit.ToolbarSeparator());
		}

		// Add select and deselect button
		if (this.multiSelect)
		{

			button = new ibm.tivoli.tip.dijit.TIPButton(
				{	iconClass: "selectAllIcon",
					label: this.resources_.SELECT_ALL,
					showLabel: false,
				 	onClick: dojo.hitch (this, this.selectAll) 
				});
			this.toolbar.addChild(button);
			
			button = new ibm.tivoli.tip.dijit.TIPButton(
				{	iconClass: "deselectAllIcon",
					label: this.resources_.DESELECT_ALL,
					showLabel: false,
				 	onClick: dojo.hitch (this, this.deselectAll) 
				});
			this.toolbar.addChild(button);
		}
		
		var addSeparator = false;
		// Add quick action buttons
	    if (this.createButton)
	    {
			this.toolbar.addChild (new dijit.ToolbarSeparator());
			addSeparator = true;	    
			this.toolbar.addChild(this.createButton);
		}

		if (this.deleteButton)
		{
			if ( addSeparator == false )
				this.toolbar.addChild (new dijit.ToolbarSeparator());
			this.toolbar.addChild(this.deleteButton);
		}
    
	    // Add quick action buttons
	    if (this.actionsDropDown)
	    {
			this.toolbar.addChild (new dijit.ToolbarSeparator());
			this.toolbar.addChild(this.actionsDropDown);
			this.toolbar.addChild (new dijit.ToolbarSeparator());
	    }
	},
	
	onSearch: function (tipSearchField, event) 
	{
		console.log ("enter onSearch with search value: " + tipSearchField.searchField.value);
		var columns = tipSearchField.getSelectedColumns();
		console.log ("columns: " + columns);
		this.search (columns, tipSearchField.searchField.value + "*" );
	},
	
	search: function (columns, searchFor) {
		this.query = {};
		for (var x in columns) {
			if (x) {
				this.query[columns[x]] = searchFor;
			}
		}

		console.log ("query is: ");
		console.log (this.query);
		this.grid.filter(this.query, true);
	},

	getTable : function () {
		return this.grid;
  },
 
 	resize : function () {
 		if (this.grid)
			this.grid.resize();
  },
   
	setStore : function (store) {
		this.grid.setStore(store);
		if (this.grid.getSortIndex() > 0)
			this.grid.sort();
		this.grid.resize();
		this.grid.selection.deselectAll();
		this.updateFooter(this.grid.rowCount, 0);
	/*	
		if ( this.grid.rowCount == 0 )
		{
			var item = {};
			
		} */
	},
	
	onRefresh : function () {
		console.log("fbfwca.Grid: onRefresh");
		if (this.multiSelect)
			this.gridParms.store = fbfwca.ajax.getJsonWriteStore(this.storeURL);
		else
			this.gridParms.store = fbfwca.ajax.getJsonReadStore(this.storeURL);
		this.setStore(this.gridParms.store);
	
	},

	/**
	 * selectAll()
	 *
	 * Select all rows
	 */
	selectAll : function () {
		var rowcnt = this.grid.rowCount;
    
		for (i=0; i<rowcnt; i++)
		{
			this.grid.selection.addToSelection(i);
		}
		
		// In order for this to work, _checked needs to be initialized to a false boolean.
		var grid = this.grid;
		this.grid.store.fetch(
			{
				query: {_checked:false},
				onItem: function(item, request) {
						grid.store.setValue(item, "_checked", true);
				}
			}
		);

		this.updateFooter ( rowcnt, rowcnt );
	},

	/**
	 * deselectAll()
	 *
	 * Deselect all rows
	 */
	deselectAll : function () {
		this.grid.selection.deselectAll();
		
		// In order for this to work, _checked needs to be initialized to a true boolean.
		var grid = this.grid;
		this.grid.store.fetch(
			{
				query: {_checked:true},
				onItem: function(item, request) {
						grid.store.setValue(item, "_checked", false);
				}
			}
		);
				
		this.updateFooter ( this.grid.rowCount, 0 );
	},

	/**
	 * updateFooter()
	 *
	 * Update table footer info
	 */
	updateFooter : function (totalRows, numSelected) {
		var footerText = null;
		if (this.multiSelect || this.filterOn)
			footerText = dojo.string.substitute(this.NLS.T_TABLE_FOOTER_TEXT, { 0: numSelected, 1: totalRows });
		else
			footerText = dojo.string.substitute(this.NLS.T_TABLE_FOOTER_TEXT_READONLY, { 0: totalRows });
		this.gridRowCount.innerHTML = footerText;
	},

	/**
	 * onSelectionChanged()
	 *
	 * Selections are based on the checkboxes.  Check on the checkbox will select or deselect the row.
	 * When clicking on other cells, the logic is as followed (similar to yahoo message box):
	 *		1) single row has been selected and the same row is selected will preserve the selection
	 *		2) single row has been selected and a different row is selected, the row is deselected
	 *		3) multiple rows have been selected and any row is now all rows will be deselected
	 */
	onSelectionChanged : function (e) {
		if (e.grid.getCell(e.cellIndex).field == "name")
		{
			return;
		}
		
		var	row = e.grid.getItem(e.rowIndex);

		// Case when checkbox column is selected
	    if (e.grid.getCell(e.cellIndex).field == "_checked")
	    {
			// Deselect row when checkbox=false
			//if (e.grid.store.getValue(row, "_checked", "0") == false )
			if (e.grid.store.getValue(row, "_checked") == false )
					e.grid.selection.deselect(e.rowIndex);
	
			// Reselect all rows that have been selected because Dojo unselects them
			e.grid.store.fetch(
			{
				query: {_checked:true},
				onItem: function(item, request) {
						e.grid.selection.addToSelection(item);
				}
			});
	    }
	
		// Case when other cells in the table is selected
	    else
	    {
			// Dojo overrides the selections, we need to count the checkboxes
			var count = 0;
			e.grid.store.fetch(
			{
				query: {_checked:true},
				onItem: function(item, request) {
						count++;
				}
			});

			// Single row selection and the same row, make sure it is still selected
			if (count == 1 && 
					(e.grid.store.getValue(row, "_checked") == true ) )
//					(e.grid.store.getValue(row, "_checked", "0") == true ) )
			{
					e.grid.selection.addToSelection(e.rowIndex);
			}

			// Multiple row selections or single row with different selected now are all deslected
			else
			{
					e.grid.selection.deselect(e.rowIndex);
					e.grid.store.setValue(row, "_checked", false);
					e.grid.store.fetch(
					{
						query: {_checked:true},
						onItem: function(item, request) {
								e.grid.selection.deselect(item);
								e.grid.store.setValue(item, "_checked", false);
						}
					});
			}
		}

		this.updateFooter (e.grid.rowCount, e.grid.selection.getSelectedCount()); 
	},

    /**
     * selectIndex()
     *
     *
     */
    selectIndex : function (index) {
        if (index < 0 )
        {
            console.log("undefined index: "+ index);
            return;
        }

        var	row = this.grid.getItem(index);

        var grid = this.grid;
        console.log("set selection for index: "+ index);
        grid.store.setValue(row, "_checked", true);
        /*grid.store.fetch(
            {
                query: {_checked:false},
                onItem: function(item, request) {
                        grid.store.setValue(item, "_checked", true);
                }
            }
        ); */

        this.updateFooter ( this.grid.rowCount, 1 );
        
    }

});

}
