/*
	Copyright (c) 2004-2009, The Dojo Foundation All Rights Reserved.
	Available via Academic Free License >= 2.1 OR the modified BSD license.
	see: http://dojotoolkit.org/license for details
*/


/*
	Copyright (c) 2004-2009, The Dojo Foundation All Rights Reserved.
	Available via Academic Free License >= 2.1 OR the modified BSD license.
	see: http://dojotoolkit.org/license for details
*/


if(!dojo._hasResource["ibm.tivoli.tip.dijit.ListBuilder"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.ListBuilder"] = true;
/**
 * ibm.tivoli.tip.dijit.ListBuilder is a pair of ibm.tivoli.tip.dijit.DndListboxes, with Add and Remove buttons between them
 * 
 * Creating a ListBuilder in markup:
 * <div dojoType="ibm.tivoli.tip.dijit.ListBuilder" showAllButtons="true" filtered="true" sorted="false" accept="role" 
 * 		type="role,text" name="myroles" leftLabel="Available roles" rightLabel="Selected roles"></div>
 * 
 * Attributes:
 * shoAllButtons: if true, show the "Add All" and "Remove All" buttons
 * sorted: if true, the right-hand list is kept sorted. the left-hand list is always sorted
 * filtered: if true, the lists include the filter textbox
 * accept: array of types that can be dropped on the 2 DndListboxes
 * type: array of types describing the items in the 2 DndListboxes
 * leftLabel: text displayed above the left-hand list
 * rightLabel: text displayed above the right-hand list
 * 
 * Events:
 * 	fires OnChange when the value of the list changes. (TODO: implement priorityChange to only fire when list looses focus)
 * 
 */
dojo.provide("ibm.tivoli.tip.dijit.ListBuilder");

dojo.require("dijit.form.Button");
dojo.require("ibm.tivoli.tip.dijit.DndListbox");
dojo.require("dojo.i18n");
dojo.requireLocalization("ibm.tivoli.tip.dijit", "resources", null, "ROOT,ar,cs,da,de,en,en-us,es,fi,fr,he,hr,hu,it,ja,ko,nl,no,pl,pt-br,ro,ru,sl,sv,tr,zh-cn,zh-tw", "ROOT,ar,cs,da,de,en,en-us,es,fi,fr,he,hr,hu,it,ja,ko,nl,no,pl,pt-br,ro,ru,sl,sv,tr,zh-cn,zh-tw");

dojo.declare("ibm.tivoli.tip.dijit.ListBuilder", [dijit._Widget, dijit._Templated],
{
	widgetsInTemplate: true,
	templateString:"<div class=\"tip-ListBuilder\">\r\n  <div dojoAttachPoint=\"left\" class=\"balLayoutCell\" dojoType=\"ibm.tivoli.tip.dijit.DndListbox\" \r\n  \t\tfiltered=\"${filtered}\" sorted=\"true\" accept=\"${accept}\" type=\"${type}\" label=\"${leftLabel}\"></div>\r\n  <div class=\"balLayoutCell\">\r\n    <div class=\"balBtnContainer\">\r\n      <button dojoType=\"dijit.form.Button\" dojoAttachPoint=\"_balAddBtn\" dojoAttacheEvent=\"onclick: add\" >${addLabel} &gt;</button>\r\n      <br/>\r\n      <button dojoType=\"dijit.form.Button\" dojoAttachPoint=\"_balAddAllBtn\" dojoAttacheEvent=\"onclick: addAll\">${addAllLabel} &gt;</button>\r\n      <br/>\r\n      <button dojoType=\"dijit.form.Button\" dojoAttachPoint=\"_balRemBtn\" dojoAttacheEvent=\"onclick: remove\">&lt; ${removeLabel}</button>\r\n      <br/>\r\n      <button dojoType=\"dijit.form.Button\" dojoAttachPoint=\"_balRemAllBtn\" dojoAttacheEvent=\"onclick: removeAll\" >&lt; ${removeAllLabel}</button>\r\n    </div>\r\n  </div>\r\n  <div dojoAttachPoint=\"right\" class=\"balLayoutCell\" dojoType=\"ibm.tivoli.tip.dijit.DndListbox\"\r\n\t\tfiltered=\"${filtered}\" sorted=\"${sorted}\" accept=\"${accept}\" type=\"${type}\" label=\"${rightLabel}\" name=\"${name}\"></div>\r\n  <div style=\"clear:both;height:0;width:0\"></div>\r\n</div>\r\n",
	showAllButtons: true,	// do we show the Add All and Remove All buttons
	filtered: true,			// do we show the filter input text boxes
	sorted: false,			// is the right list kept sorted? (the left list is always sorted)
	accept: ["text"],		// array of types that can be dropped here
	type: ["text"],			// array of types objects in this box represent
	name: "",				// give the widget a name if you want its value submitted with a form
	leftLabel: "",			// label for the left list
	rightLabel: "",			// label for the right list
	required: false,		// is a value required for this list?
	// nls strings
	addLabel: "",
	removeLabel: "",
	addAllLabel: "",
	removeAllLabel: "",

	postMixInProperties: function()
	{
		var string_table = dojo.i18n.getLocalization("ibm.tivoli.tip.dijit", "resources");
		this.addLabel = string_table.ADD;
		this.removeLabel = string_table.REMOVE;
		this.addAllLabel = string_table.ADDALL;
		this.removeAllLabel = string_table.REMOVEALL;
		this.inherited(arguments);
	},
	/**
	 * startup()
	 *
	 * Basic start-up rountine, connect event handlers and 
	 * add the first items to the list.
	 */
	startup: function() {
		if(!this.showAllButtons) {
			dojo.style(this._balAddAllBtn.domNode, "display", "none");
			dojo.style(this._balRemAllBtn.domNode, "display", "none");
		}

		// Connect click listeners to the buttons
		this.connect(this._balAddBtn.domNode, 'onclick', this.add);
		this.connect(this._balAddAllBtn.domNode, 'onclick', this.addAll);
		this.connect(this._balRemBtn.domNode, 'onclick', this.remove);
		this.connect(this._balRemAllBtn.domNode, 'onclick', this.removeAll);
		//dojo.subscribe("/dnd/drop", dojo.hitch(this, this.updateButtons));
		this.connect(this.domNode, 'onclick', this.updateButtons);
		
		this.connect(this.left, "OnChange", this.updateButtons);
		this.connect(this.right, "OnChange", this._onChange);
		
		this.updateButtons();
		this.inherited(arguments);
	},
	destroy: function()
	{
		console.log("ListBuilder.destroy");
		this.inherited(arguments);
	},
	_getValueAttr: function()
	{
		var list = this.right.attr("value");
		return list;
	},
	_setValueAttr: function(value)
	{
		var item;
		var items = [];
		var varr = dojo.isArray(value) ? value : [value];
		var from = this.left;
		var to = this.right;
		
		// move non-matching items from the right list back to the left
		to.getAllNodes().forEach(function(n){
			item = to.getItem(n.id);
			if(item && item.data && dojo.indexOf(varr, item.data.toString()) < 0) {
				items.push(item.data);
				to.delItem(n.id);
			}
		});
		from.insertNodes(items);
		
		// move matching items from the left list to the right
		items = [];
		from.getAllNodes().forEach(function(n){
			item = from.getItem(n.id);
			if(item && item.data && dojo.indexOf(varr, item.data.toString()) >= 0) {
				items.push(item.data);
				from.delItem(n.id);
			}
		});
		this.right.insertNodes(items);

		this.updateButtons();
	},
	/**
	 * addData()
	 * 
	 * Add data to the available list
	 * 
	 * Input:
	 * 	data: the data being added to the available list
	 * 
	 */
	addData: function(data)
	{
		this.left.addData(data);
		this.updateButtons();
	},
	
	/**
	 * addAll()
	 *
	 * Add All button handler.  Moves all items from the available list to 
	 * the reserve list.
	 * 
	 * Inputs:
	 *   e- DOM event object from Dojo (assumed fixed)
	 * Returns:
	 *   None.
	 */
	addAll: function(e) {
		this.left.selectAll();
		this._move(this.left, this.right);
		this.updateButtons();
	},

	/**
	 * add()
	 *
	 * Add button handler.  Moves selected items from the left list to 
	 * the right list.
	 * 
	 * Inputs:
	 *   e- DOM event object from Dojo (assumed fixed)
	 * Returns:
	 *   None.
	 */
	add: function(e) {
		this._move(this.left, this.right);
	},

	/**
	 * removeAll()
	 *
	 * Remove All button handler.  Moves all items reserve from the list to 
	 * the available list.
	 * 
	 * Inputs:
	 *   e- DOM event object from Dojo (assumed fixed)
	 * Returns:
	 *   None.
	 */
	removeAll: function(e) {
		 this.right.selectAll();
		 this._move(this.right, this.left);
		 this.updateButtons();
	},
	/**
	 * remove()
	 *
	 * Remove button handler.  Moves selected items from the right list
	 * to the left list.
	 * 
	 * Inputs:
	 *   e- DOM event object from Dojo (assumed fixed)
	 * Returns:
	 *   None.
	 */
	remove: function(e) {
		this._move(this.right, this.left);
	},
	_move: function(from, to)
	{
		var nodes = from.getSelectedNodes();
		// for some reason dojo.dnd can do it, but I can't insert nodes, 
		// but only items.  This also means I loose the items' type in 
		// the process
		var items = [];
		var item;
		nodes.forEach(function(n) {
			item = from.getItem(n.id);
			if(item && item.data)
			items.push(item.data);
		});
		to.insertNodes(items);
		from.deleteSelected();
		this.updateButtons();
	},
	/**
	 * updatesButtons()
	 *
	 * Updates all buttons disabled state based on the content
	 * of the list.  It disables button when the list is empty 
	 * and enable button when their is something to move.
	 * 
	 * Inputs:
	 *   None.
	 * Returns:
	 *   None.
	 */
	updateButtons: function() {
		console.debug(this.id + ".updateButtons");

		// If any nodes are selected, enable the add button
		var b = this.left.getSelectedNodes().length < 1;
		this._balAddBtn.attr('disabled', b);

		// If the available area contains any nodes, enable the add all button
		b = this.left.attr("value").length < 1;
		this._balAddAllBtn.attr('disabled', b);

		// If the reserve area contains any seelected items, enable the remove button
		b = this.right.getSelectedNodes().length < 1;
		this._balRemBtn.attr('disabled', b);

		// If the available area contains any nodes, enable the remove all button
		b = this.right.attr("value").length < 1;
		this._balRemAllBtn.attr('disabled', b);
	},
	isValid: function()
	{
		return !this.required || this.attr("value").length > 0;
	},
	_onChange: function()
	{
		this.updateButtons();
		this.OnChange();
	},
	OnChange: function()
	{
		// override or connect to me
	}
});




}
